#!/usr/bin/env python
from numpy import *
from matplotlib.pyplot import *

# England Formel
bs_A = array([[0.0,  0.0, 0.0, 0.0],
              [2.0,  0.0, 0.0, 0.0],
              [1.0,  1.0, 0.0, 0.0],
              [0.0, -4.0, 8.0, 0.0]]) / 4.0

bs_b = array([1.0, 0.0, 4.0, 1.0]) / 6.0

bs_c = array([0.0, 0.5, 0.5, 1.0])


# ex. a

def RK_step(rhs, A, b, c, t, y, h):
    """
    Makes a single Runge-Kutta step of size h.

    Input:
    rhs: right hand side of considered differential eq.
    A, b, c: Butcher Schema
    t: Current time t_i
    y: Current solution y_i
    h: Timestep size
    """
    tnew = t
    ynew = y
    s = A.shape[0]
    k = zeros((2, s))

    #############################################
    #                                           #
    # TODO: Implementieren Sie einen RK-Schritt #
    #                                           #
    #############################################

    tnew = t + h
    ynew = y + h * squeeze(dot(k, b))
    return tnew, ynew


def RK(rhs, A, b, c, y0, tstart, tend, N):
    """
    Integrate the equation with a Runge-Kutta.

    Input:
    rhs: right hand side of considered differential eq.
    A, b, c: Butcher Schema
    y0: initial condition
    tstart, tend: start, end time
    N: number of steps
    """
    y = zeros((2, N+1))
    t = zeros((N+1,))
    t[0] = tstart
    y[:,0] = y0
    h = (tend - tstart) / float(N)
    for n in xrange(N):
        t[n+1], y[:,n+1] = RK_step(rhs, A, b, c, t[n], y[:,n], h)
    return t, y


# ex b

IV = array([0.0, 1.0])
T = 10.0

##################################################################
#                                                                #
# TODO: Numerisches Experiment, um die Konvergenzrate zu erraten #
#                                                                #
##################################################################
Ns = array([])
hs = array([])
listErrors = []

k = 0  # Schreiben Sie den richtigen Konvergenzordnung

figure(figsize=(14, 10))
loglog(hs, listErrors, 'x-', label='Fehler')
loglog(hs, hs**k, label='$h^{:d}$'.format(int(k)))
xlabel('Fehler')
xlabel('Schrittweite')
legend()
grid(True)
savefig('RK_EnglandFehler.png')


# ex c und d

IV = array([2.0, 95.0])
T = 1.0

############################################################
#                                                          #
# TODO: Finden und ploten Sie hier die gefragten Loesungen #
#       Schreiben Sie die Erklaerungen auf Papier !        #
#                                                          #
############################################################

figure(figsize=(14, 10))
legend()
xlabel('Zeit')
ylabel('Loesung')
xlim(0.0, T)
grid(True)
savefig('RK_EnglandLoesung.png')


# ex e

from numpy.linalg import det


def S(z):
    result = z
    ##########################################################
    #                                                        #
    # TODO: Implementieren Sie hier die Stabilitaetsfunktion #
    #       Schreiben Sie Herleitung und Wert auf Papier !   #
    #                                                        #
    ##########################################################
    return result


xl = -5.0
xr = 3.0
yl = -4.0
yr = 4.0
xd = linspace(xl, xr, 200)
yd = linspace(yl, yr, 200)
X, Y = meshgrid(xd, yd)
Z = X + 1j * Y
Sz = array(map(S, Z.flat)).reshape(Z.shape)

figure(figsize=(6, 6))
contourf(X, Y, where(abs(Sz) <= 1.0, 1.0, -inf), levels=linspace(0, 2, 51))
contour(X, Y, abs(Sz), levels=[1.0], colors='k', linewidths=[2])
grid(True)
xlim(-5, 3)
ylim(-4, 4)
xlabel(r'$\Re z$')
ylabel(r'$\Im z$')
savefig('RK_EnglandStabilityDomain.png')
