#!/usr/bin/env python
from numpy import *
from numpy.linalg import *
import scipy.optimize
from matplotlib.pyplot import *


# Unteraufgabe a)

def F(q, p, m, D=3):
    """The function f(y)

    Input: q ...  array with positions
           p ...  array with momenta
           m ...  array with masses
           D ...  space dimension, default equals 3.

    Output: dq ... time-derivative of the positions
            dp ... time-derivative of the momenta
    """
    
    
    N = q.size // D
    # Empty arrays for computed data
    dq = zeros_like(p)
    dp = zeros_like(q)
    #######################################################
    #                                                     #
    # TODO: Implementieren Sie hier die rechte Seite f(y) #
    #       wobei y = (q, p) wie in der Beschreibung.     #
    #                                                     #
    #######################################################
    
    return dq, dp


def rhs(y):
    """Right hand side

    Input: y ... array of q and p:  y = (q, p)

    Output: dy ... time-derivative of y
    """
    q, p = hsplit(y, 2)
    dy = hstack(F(q, p, m))
    return dy




# Unteraufgabe b)

def integrate_EE(y0, xStart, xEnd, steps, flag=False):
    r"""Integrate ODE with explicit Euler method

    Input: y0     ... initial condition
           xStart ... start x
           xEnd   ... end   x
           steps  ... number of steps (h = (xEnd - xStart)/N)
           flag   ... flag == False return complete solution: (phi, phi', t)
                      flag == True  return solution at endtime only: phi(tEnd)

    Output: x ... variable
            y ... solution
    """
    
    h = (xEnd - xStart) / (1.0*steps)
    x = zeros(steps+1)
    y = zeros((steps+1, size(y0)))
    ###########################################################
    #                                                         #
    # TODO: Implementieren Sie hier die explizite Euler Regel #
    #       zur integration der funktion y(x).                #
    #                                                         #
    ###########################################################
    if flag:
        return x[-1], y[-1][:]
    else:
        return x, y


def integrate_IE(y0, xStart, xEnd, steps, flag=False):
    r"""Integrate ODE with implicit Euler method

    Input: y0     ... initial condition
           xStart ... start x
           xEnd   ... end   x
           steps  ... number of steps (h = (xEnd - xStart)/N)
           flag   ... flag == False return complete solution: (phi, phi', t)
                      flag == True  return solution at endtime only: phi(tEnd)

    Output: x ... variable
            y ... solution
    """
    
    h = (xEnd - xStart) / (1.0*steps)
    x = zeros(steps+1)
    y = zeros((steps+1, size(y0)))
    ###########################################################
    #                                                         #
    # TODO: Implementieren Sie hier die implizite Euler Regel #
    #       zur integration der funktion y(x).                #
    #                                                         #
    ###########################################################
    if flag:
        return x[-1], y[-1][:]
    else:
        return x, y


def integrate_IM(y0, xStart, xEnd, steps, flag=False):
    r"""Integrate ODE with implicit midpoint rule

    Input: y0     ... initial condition
           xStart ... start x
           xEnd   ... end   x
           steps  ... number of steps (h = (xEnd - xStart)/N)
           flag   ... flag == False return complete solution: (phi, phi', t)
                      flag == True  return solution at endtime only: phi(tEnd)

    Output: x ... variable
            y ... solution
    """

    h = (xEnd - xStart) / (1.0*steps)
    x = zeros(steps+1)
    y = zeros((steps+1, size(y0)))
    #################################################################
    #                                                               #
    # TODO: Implementieren Sie hier die implizite Mittelpunktsregel #
    #       zur integration der funktion y(x).                      #
    #                                                               #
    #################################################################
    if flag:
        return x[-1], y[-1][:]
    else:
        return x, y


# Unteraufgabe c)
        
def rhs_vv(q, D=3):
    """Right hand side

    Input: q ...  array with positions

    Output: dp ... time-derivative of the velocities
    """
    
    N = q.size // D
    # Empty arrays for computed data
    dp = zeros_like(q)
    #######################################################
    #                                                     #
    # TODO: Implementieren Sie hier die rechte Seite f(y) #
    #       geeignet fuer die velocity Verlet Methode.    #
    #                                                     #
    #######################################################
    return dp


def integrate_VV(y0, xStart, xEnd, steps, flag=False):
    r"""Integrate ODE with velocity verlet rule

    Input: y0     ... initial condition
           xStart ... start x
           xEnd   ... end   x
           steps  ... number of steps (h = (xEnd - xStart)/N)
           flag   ... flag == False return complete solution: (phi, phi', t)
                      flag == True  return solution at endtime only: phi(tEnd)

    Output: x ... variable
            y ... solution
    """
    
    h = (xEnd - xStart) / (1.0*steps)
    x = zeros(steps+1)
    y = zeros((steps+1, size(y0)))
    #############################################################
    #                                                           #
    # TODO: Implementieren Sie hier die velocity Verlet Methode #
    #       zur integration der funktion y(x).                  #
    #                                                           #
    #############################################################
    if flag:
        return x[-1], y[-1][:]
    else:
        return x, y


# Tests for two bodies
G = 1.0
m = array([500.0, 1.0])
q0 = hstack([0,0,0, 2,0,0])
p0 = hstack([0,0,0, 0,sqrt(m[0]/q0[3]),0])
y0 = hstack([q0, p0])

# Compute
T = 3
nrsteps = 5000

t_ee, y_ee = integrate_EE(y0, 0, T, nrsteps, False)
t_ie, y_ie = integrate_IE(y0, 0, T, nrsteps, False)
t_im, y_im = integrate_IM(y0, 0, T, nrsteps, False)
t_vv, y_vv = integrate_VV(y0, 0, T, nrsteps, False)

# Plot
fig = figure(figsize=(12,8))
ax = fig.gca()
ax.set_aspect("equal")
ax.plot(y_ee[:,0], y_ee[:,1], "b-")
ax.plot(y_ee[:,3], y_ee[:,4], "b-", label="EE")

ax.plot(y_ie[:,0], y_ie[:,1], "g-")
ax.plot(y_ie[:,3], y_ie[:,4], "g-", label="IE")

ax.plot(y_im[:,0], y_im[:,1], "r-")
ax.plot(y_im[:,3], y_im[:,4], "r-", label="IM")

ax.plot(y_vv[:,0], y_vv[:,1], "m-")
ax.plot(y_vv[:,3], y_vv[:,4], "m-", label="VV")

ax.grid(True)
ax.set_xlim(-3,3)
ax.set_ylim(-3,3)
ax.legend(loc="upper right")
ax.set_xlabel("x")
ax.set_ylabel("y")
fig.savefig("zwei.pdf")
